
// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
// Autonomous vehicle source code - v1.0 - 20/11/2015
//
// AUTHOR/LICENSE:
// Created by Umberto Cilia - www.umbertocilia.it
// Copyright 2014 License: GNU GPL v3 http://www.gnu.org/licenses/gpl-3.0.html
//
// LINKS:
// 
// 
//
// DISCLAIMER:
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// BACKGROUND:
// This is a first approach to arduino embedded systems, using NewPing and LedControlMS library and some other 
// custom functions to drive a two  dc motors robot.
// FEATURES:
// * Sensors = hc-sr04 ultrasonic sensor.
// * Output interface = 8x8 led lcd matrix display.
// * Actuators = 2 dc motors.
// * Motor driver = h bridge sn 754410.
// * Controller unit = arduino uno.


// SYNTAX:
// front_check();  return 1 if there is an obstale at 15 cm or 0 in the opposite case and display on led matrix a radar signal for 15 25 or infinite ostacles.
// right_check();  return 1 if there is an obstale at 15 cm or 0 in the opposite case and display on led matrix a radar signal for 15 25 or infinite ostacles.
// left_check();  return 1 if there is an obstale at 15 cm or 0 in the opposite case and display on led matrix a radar signal for 15 25 or infinite ostacles.
// go_forwards(); start moving forwards.
// go_backwards(); start moving backwards.
// rotate_left();     rotate left.
// rotate_right();  rotate right;
// stop_move();     arrests the robot.
// ---------------------------------------------------------------------------


// include NewPing.h to controll ultrasonic sensors
// include LedControlMS.h to controll 8x8 led matrix

#include <NewPing.h>
#include "LedControlMS.h"

// setting up 8x8 led matrix pins numbers

LedControl lc=LedControl(12,11,10,1);

// define motors pin connected to sn754410 and arduino.

#define R_motor_forwards  A2
#define R_motor_backwards 5

#define L_motor_forwards  A1
#define L_motor_backwards  A0

#define Power_Control 3

#define TRIGGER_FRONT 4
#define ECHO_FRONT    6

#define TRIGGER_RIGHT  9
#define ECHO_RIGHT     2

#define TRIGGER_LEFT    8
#define ECHO_LEFT       7

#define MAX_DISTANCE 200

// setting up motors power level .

int Power_Level = 200;


// initialize sensors check variables.

boolean check_front = 0;
boolean check_right = 0;
boolean check_left = 0;


// Define 8x8 led matrix patterns.

byte center[8]={B00000000,B00000000,B00011000,B00111100,B00111100,B00011000,B00000000,B00000000};

byte front_max[8]={B00000000,
                   B00000000,
                   B00000000,
                   B00000000,
                   B00000000,
                   B00011000,
                   B00111100,
                   B01111110};



byte fmedium[8]={B00000000,
                 B00000000,
                 B00000000,
                 B00000000,
                 B00000000,
                 B00011000,
                 B00111100,
                 B00000000};

byte lmax[8]={B00000000,
              B00000001,
              B00000011,
              B00001111,
              B00001111,
              B00000011,
              B00000001,
              B00000000};

byte lmedium[8]={B00000000,
                 B00000000,
                 B00000010,
                 B00001110,
                 B00001110,
                 B00000010,
                 B00000000,
                 B00000000};

byte rmax[8]={B00000000,
              B10000000,
              B11000000,
              B11110000,
              B11110000,
              B11000000,
              B10000000,
              B00000000};

byte rmedium[8]={B00000000,
                 B00000000,
                 B01000000,
                 B01110000,
                 B01110000,
                 B01000000,
                 B00000000,
                 B00000000};








void setup() {

  lc.shutdown(0,false);
  lc.setIntensity(0,8);
  lc.clearDisplay(0);
  
 
  pinMode(R_motor_forwards, OUTPUT);
  pinMode(R_motor_backwards, OUTPUT);
  pinMode(L_motor_forwards, OUTPUT);
  pinMode(L_motor_backwards, OUTPUT);
  pinMode(Power_Level, OUTPUT);

  analogWrite(Power_Control, Power_Level);
  


}

void loop() {

 


    
    check_front = front_check();
    delay(50);
    check_right = right_check();
    delay(50);
    check_left = left_check();

    if( check_front == 0 && check_right == 0 && check_left == 0){
      
            go_forwards();
      
      
      }else{
        
              stop_move();

             if(check_front == 1 && check_left == 1 && check_right== 1){

              go_backwards();
              delay(500);
              stop_move();
              rotate_right();
              delay(500);
              stop_move();
               go_forwards();
             
               }else if(check_front == 1 && check_left == 0 && check_right== 1){

             
              stop_move();
              rotate_right();
              delay(200);
              stop_move();
              go_forwards();
               }else if(check_front == 1 && check_left == 1 && check_right== 0){

              
              stop_move();
              rotate_left();
              delay(200);
              stop_move();
              go_forwards();
               }else if(check_front == 1 && check_left == 0 && check_right== 0){

              go_backwards();
              delay(500);
              stop_move();
              rotate_left();
              delay(500);
              stop_move();
              go_forwards();
               }else if(check_front == 0 && check_left == 1 && check_right== 0){

              
              stop_move();
              rotate_right();
              delay(200);
              stop_move();
              go_forwards();
               }else if(check_front == 0 && check_left == 0 && check_right== 1){

              
              stop_move();
              rotate_left();
              delay(200);
              stop_move();
              go_forwards();
               }
               else if(check_front == 0 && check_left == 1 && check_right== 1){

              
              stop_move();
              go_backwards();
              delay(700);
              stop_move();
              rotate_left();
              delay(1000);
              stop_move();
              go_forwards();
               }
              
              
              }
                
                
                
              
              
        
               








}



















boolean front_check() {

  //verifica che non ci siano ostacoli di fronte 1 se positivo

  int distance = 0;
  boolean check = 0;

  NewPing sonar(TRIGGER_FRONT, ECHO_FRONT, MAX_DISTANCE);

  distance = sonar.ping_cm();

   if(distance>25){
    
    
    center_led();
    
    }

    if(distance<25 && distance>15){
      
        lc.setRow(0,0,fmedium[0]);
        lc.setRow(0,1,fmedium[1]);
        lc.setRow(0,2,fmedium[2]);
        lc.setRow(0,3,fmedium[3]);
        lc.setRow(0,4,fmedium[4]);
        lc.setRow(0,5,fmedium[5]);
        lc.setRow(0,6,fmedium[6]);
        lc.setRow(0,7,fmedium[7]);

       

        lc.clearDisplay(0);
      
      
      }
  

  if (distance > 15) {

    check = 0;

  } else {

        lc.setRow(0,0,front_max[0]);
        lc.setRow(0,1,front_max[1]);
        lc.setRow(0,2,front_max[2]);
        lc.setRow(0,3,front_max[3]);
        lc.setRow(0,4,front_max[4]);
        lc.setRow(0,5,front_max[5]);
        lc.setRow(0,6,front_max[6]);
        lc.setRow(0,7,front_max[7]);
        
       

        lc.clearDisplay(0);

    check = 1;

  }



  return check;

}













boolean right_check() {

  //verifica che non ci siano ostacoli a destra 1 se positivo

  int distance = 0;
  boolean check = 0;

  NewPing sonar(TRIGGER_RIGHT, ECHO_RIGHT, MAX_DISTANCE);

  distance = sonar.ping_cm();

   if(distance>25){
    
    
    center_led();
    
    }

    if(distance<25 && distance>15){
      
        lc.setRow(0,0,rmedium[0]);
        lc.setRow(0,1,rmedium[1]);
        lc.setRow(0,2,rmedium[2]);
        lc.setRow(0,3,rmedium[3]);
        lc.setRow(0,4,rmedium[4]);
        lc.setRow(0,5,rmedium[5]);
        lc.setRow(0,6,rmedium[6]);
        lc.setRow(0,7,rmedium[7]);

        
        lc.clearDisplay(0);
      
      
      }

  if (distance > 15) {

    check = 0;

  } else {

        lc.setRow(0,0,rmax[0]);
        lc.setRow(0,1,rmax[1]);
        lc.setRow(0,2,rmax[2]);
        lc.setRow(0,3,rmax[3]);
        lc.setRow(0,4,rmax[4]);
        lc.setRow(0,5,rmax[5]);
        lc.setRow(0,6,rmax[6]);
        lc.setRow(0,7,rmax[7]);
        
       

        lc.clearDisplay(0);

    check = 1;

  }



  return check;

}










boolean left_check() {

  //verifica che non ci siano ostacoli a sinistra 1 se positivo
  int distance = 0;
  boolean check = 0;

  NewPing sonar(TRIGGER_LEFT, ECHO_LEFT, MAX_DISTANCE);

  distance = sonar.ping_cm();

  if(distance>25){
    
    
    center_led();
    
    }

    if(distance<25 && distance>15){
      
        lc.setRow(0,0,lmedium[0]);
        lc.setRow(0,1,lmedium[1]);
        lc.setRow(0,2,lmedium[2]);
        lc.setRow(0,3,lmedium[3]);
        lc.setRow(0,4,lmedium[4]);
        lc.setRow(0,5,lmedium[5]);
        lc.setRow(0,6,lmedium[6]);
        lc.setRow(0,7,lmedium[7]);

        

        lc.clearDisplay(0);
      
      
      }

     
      

  if (distance > 15) {

        

    check = 0;

  } else {

        lc.setRow(0,0,lmax[0]);
        lc.setRow(0,1,lmax[1]);
        lc.setRow(0,2,lmax[2]);
        lc.setRow(0,3,lmax[3]);
        lc.setRow(0,4,lmax[4]);
        lc.setRow(0,5,lmax[5]);
        lc.setRow(0,6,lmax[6]);
        lc.setRow(0,7,lmax[7]);
        
        

        lc.clearDisplay(0);

    check = 1;

  }



  return check;


}


void go_forwards() {

  digitalWrite(R_motor_forwards, HIGH);
  digitalWrite(L_motor_forwards, HIGH);


}


void go_backwards() {

  digitalWrite(R_motor_backwards, HIGH);
  digitalWrite(L_motor_backwards, HIGH);

}

void rotate_right() {

  digitalWrite(R_motor_backwards, HIGH);
  digitalWrite(L_motor_forwards, HIGH);


}


void rotate_left() {

  digitalWrite(R_motor_forwards, HIGH);
  digitalWrite(L_motor_backwards, HIGH);



}


void stop_move() {


  digitalWrite(R_motor_forwards, LOW);
  digitalWrite(L_motor_forwards, LOW);
  digitalWrite(R_motor_backwards, LOW);
  digitalWrite(L_motor_backwards, LOW);


}


void center_led(){

  
        lc.setRow(0,0,center[0]);
        lc.setRow(0,1,center[1]);
        lc.setRow(0,2,center[2]);
        lc.setRow(0,3,center[3]);
        lc.setRow(0,4,center[4]);
        lc.setRow(0,5,center[5]);
        lc.setRow(0,6,center[6]);
        lc.setRow(0,7,center[7]);
  
  }


